// Guid.h --- Private GUID definition.
//
// Copyright (C) 2004, 2007 Raymond Penners <raymond@dotsphinx.com>
// All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef GUID_H
#define GUID_H

// {B6407CFF-FCB5-4883-90D2-3B4E7B2756BF}
DEFINE_GUID(CLSID_WorkraveDeskBand, 0xb6407cff, 0xfcb5, 0x4883, 0x90, 0xd2, 0x3b, 0x4e, 0x7b, 0x27, 0x56, 0xbf);

#endif
