// preinclude.h --- Patch includes
//
// Copyright (C) 2002, 2003, 2004, 2007, 2011 Raymond Penners <raymond@dotsphinx.com>
// All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef NOMINMAX
#define NOMINMAX
#endif

#undef THREAD_PRIORITY_NORMAL
#undef THREAD_PRIORITY_LOW
#undef THREAD_PRIORITY_HIGH
#undef THREAD_PRIORITY_URGENT
#undef DELETE
#undef ERROR
#undef OK

#if defined(interface)
#undef interface
#endif
