/*
 * dll_hell.h -- DLL hell checking
 *
 * Copyright (C) 2003, 2007 Raymond Penners <raymond@dotsphinx.com>
 * Adapted from Gaim, originally written by Herman Bloggs
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DLL_HELL_H
#define DLL_HELL_H

#ifdef __cplusplus
extern "C"
{
#endif

  void dll_hell_check(void);

#ifdef __cplusplus
}
#endif

#endif /* DLL_HELL_H */
