/*
 * credits.h --- Workrave credits
 *
 * Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Rob Caelers <robc@krandor.nl>
 * All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CREDITS_H
#define CREDITS_H

#include <stdlib.h>

const char *workrave_copyright = "Copyright 2001-2020 Rob Caelers & Raymond Penners";

const char *workrave_authors[] = {"Developers:",
                                  "Rob Caelers <robc@krandor.org>",
                                  "Raymond Penners <raymond@dotsphinx.com>",
                                  "Ray Satiro <raysatiro@yahoo.com>",
                                  "",
                                  "Sounds:",
                                  "Ruairi Fullam",
                                  "Pieter De Decker",
                                  NULL};

const char *workrave_translators =
  /* bg */
  "Иван Димов\n"
  /* ca */
  "Jordi Mallach\n"
  /* cs */
  "Juraj Kubelka\n"
  /* de */
  "Johannes Rohr\n"
  /* dk */
  "Christian Vejlbo\n"
  /* dk */
  "Frederik Svarre\n"
  /* el */
  "Prokopis Prokopidis\n"
  /* eo */
  "Eric lesh\n"
  /* es */
  "Pablo Rodriguez\n"
  /* es */
  "Fco. Javier Serrador\n"
  /* et */
  "Marko Väljaots\n"
  /* fi */
  "Jyri-Petteri Paloposki\n"
  /* fr 1 */
  "Thomas Basset\n"
  /* fr 2 */
  "Benjamin Siband\n"
  /* fr 3 */
  "Darialalala\n"
  /* fa */
  "Moein Gh\n"
  /* ga */
  "Seanán Ó Coistín\n"
  /* he */
  "Haggai Eran\n"
  /* hr */
  "Ivan\n"
  /* hu-1 */
  "ORY Mate\n"
  /* hu-2 */
  "Gabor Kelemen\n"
  /* id */
  "Andika Triwidada\n"
  /* it */
  "Eraldo Girardi\n"
  /* it */
  "Albano Battistella\n"
  /* ja */
  "Masanobu Yokota\n"
  /* ko */
  "김재용\n"
  /* lt */
  "Artūras Šlajus\n"
  /* nl 1 */
  "Raymond Penners\n"
  /* nl 2 */
  "Matthias Huijgen\n"
  /* no */
  "Morten Lunde\n"
  /* pl */
  "Mikolaj Machowski\n"
  /* pt_BR -1 */
  "Claudio Ferreira Filho\n"
  /* ro */
  "Alexandru Szasz\n"
  /* rs */
  "Goran Maričić\n"
  /* ru */
  "Sergey Kirkinsky\n"
  /* ru */
  "FreeWe\n"
  /* ru */
  "Igor Zinovik\n"
  /* ru */
  "Andrey Bondrov\n"
  /* ru */
  "Denis Nikolaenko\n"
  /* sk */
  "Peter Tuharsky\n"
  /* sl */
  "Vanja Cvelbar\n"
  /* sv */
  "Daniel Nylander\n"
  /* tr */
  "Enver ALTIN\n"
  /* uk */
  "Mykola Rykov\n"
  /* zh_CN */
  "Tao WEI\n"
  /* zh_CN */
  "Alick Zhao\n"
  /* zh_TW */
  "Rex Tsai\n"
  /* zh_TW */
  "Daniel YC Lin (林原志)\n"
  /* zh_TW */
  "Yuenfu Chiu\n";

#endif /* CREDITS_H */
