<?xml version="1.0" encoding="UTF-8"?>

<node name="/org/workrave/Workrave">

{% for interface in model.interfaces %}
  <interface name="{{ interface.name }}">

{% for method in interface.methods %}
    <method name="{{ method.qname }}">
      <annotation name="org.freedesktop.DBus.GLib.CSymbol" 
                  value="{{ method.csymbol }}"/>
     {% for p in method.params %}
     <arg type="{{ p.sig() }}" name="{{ p.name }}" direction="{{ p.direction }}" />
     {% endfor %}
    </method>
{% endfor %}
  
  </interface>

{% endfor %}

</node>

  
