// MacOSUtil.hh --- MacOS utility functions
//
// Copyright (C) 2007 Rob Caelers <robc@krandor.org>
// All rights reserved.
//

#ifndef MacOSUTIL_HH
#define MacOSUTIL_HH

class MacOSUtil
{
public:
  static void init();
};

#endif // MacOSUTIL_HH
