/*
 * Copyright (C) 2011, 2013 Rob Caelers <robc@krandor.nl>
 * All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define WORKRAVE_INDICATOR_SERVICE_NAME "org.workrave.Workrave"
#define WORKRAVE_INDICATOR_SERVICE_IFACE "org.workrave.AppletInterface"
#define WORKRAVE_INDICATOR_SERVICE_OBJ "/org/workrave/Workrave/UI"
#define WORKRAVE_INDICATOR_SERVICE_VERSION 1

#define WORKRAVE_INDICATOR_CORE_NAME "org.workrave.Workrave"
#define WORKRAVE_INDICATOR_CORE_IFACE "org.workrave.CoreInterface"
#define WORKRAVE_INDICATOR_CORE_OBJ "/org/workrave/Workrave/Core"

#define WORKRAVE_INDICATOR_MENU_NAME "org.workrave.Workrave"
#define WORKRAVE_INDICATOR_MENU_OBJ "/org/workrave/Workrave/Menu"

#define DBUSMENU_WORKRAVE_MENUITEM_TYPE "x-workrave-menu-item"

#define INDICATOR_MENUITEM_TYPE "indicator-item"
#define INDICATOR_MENUITEM_PROP_LABEL "indicator-label"
#define INDICATOR_MENUITEM_PROP_ICON "indicator-icon"
